@@ -416,8 +416,7 @@ gail_button_get_keybinding (AtkAction *action,
                             gint      i)
 {
   GailButton *button;
-  G_CONST_RETURN gchar *return_value = NULL;
-
+  gchar *return_value = NULL;
 
   switch (i)
     {
@@ -446,9 +445,9 @@ gail_button_get_keybinding (AtkAction *action,
 
             key_val = gtk_label_get_mnemonic_keyval (GTK_LABEL (child)); 
             if (key_val != GDK_VoidSymbol)
-              return_value = gtk_accelerator_name (key_val, 0);
+              return_value = gtk_accelerator_name (key_val, GDK_MOD1_MASK);
             g_free (button->click_keybinding);
-            button->click_keybinding = g_strdup (return_value);
+            button->click_keybinding = return_value;
           }
         break;
       }
