@@ -339,7 +339,6 @@ gail_window_ref_state_set (AtkObject *accessible)
   GtkWidget *widget;
   GtkWindow *window;
   GdkWindowState state;
-  GValue value = { 0, };
 
   state_set = ATK_OBJECT_CLASS (parent_class)->ref_state_set (accessible);
   widget = GTK_ACCESSIBLE (accessible)->widget;
@@ -361,9 +360,7 @@ gail_window_ref_state_set (AtkObject *accessible)
   if (gtk_window_get_modal (window))
     atk_state_set_add_state (state_set, ATK_STATE_MODAL);
 
-  g_value_init (&value, G_TYPE_BOOLEAN);
-  g_object_get_property (G_OBJECT (window), "resizable", &value);
-  if (g_value_get_boolean (&value))
+  if (gtk_window_get_resizable (window))
     atk_state_set_add_state (state_set, ATK_STATE_RESIZABLE);
  
   return state_set;
